"""
    YouTube to MPD
    Copyright 2014, Jeroen Doggen, jeroendoggen@gmail.com
"""


from __future__ import print_function, division  # We require Python 2.6+

import os

from settings import Settings


class YoutubeToMPD:
    """ Contains all the tools to convert youtube content to MPD content"""
    time_passed = 0
    errors = 0
    download_counter = 0

    def __init__(self):
        self.settings = Settings()
        #self.setup = Setup(self.settings, self.logger)
        self.errors = 0

    def run(self):
        """ Run the program (call this from main) """
        self.print_start_info()
        self.convert()
        self.print_end_info()

    def convert(self):
        self.go_to_folder()
        self.download()

    def go_to_folder(self):
        if "~/" in self.settings.music_folder:
            self.settings.music_folder = self.settings.music_folder[2:]
        path = os.path.join(os.environ['HOME'], self.settings.music_folder)
        print(path)
        if (os.path.isdir(path)):
            os.chdir(path)
            path = os.path.join(path, self.settings.youtube_foldername)
            if (os.path.isdir(path)):
                os.chdir(self.settings.youtube_foldername)
                print(os.getcwd())
            else:
                print("Error opening YouTube folder")
                exit(0)
                #TODO create this folder when needed?
        else:
            print("Error opening music folder")
            exit(0)

    def download(self):
        if(self.settings.playlist) is not None:
            print("Starting playlist download")
            os.system("youtube-dl -cit --extract-audio --audio-format aac  http://www.youtube.com/playlist?list=" + self.settings.playlist)
        if(self.settings.song) is not None:
            print("Starting song download")
            os.system("youtube-dl -cit --extract-audio --audio-format aac  https://www.youtube.com/watch?v=" + self.settings.song)

    def print_start_info(self):
        print("Starting Youtube To MPD")
        print(" - Music folder: " + self.settings.music_folder)
        print(" - YouTube folder: " + self.settings.youtube_foldername)
        print(" - Create per-playlist folder: " + self.settings.create_subfolders)
        print(" - Create MPD playlist: " + self.settings.create_playlists)

    def print_end_info(self):
        print("Processing finished")
        print(" - Files downloaded: " + str(self.download_counter))
        print(" - Files converted: " + str(self.download_counter - self.errors))
        print(" - Time elapsed: " + str(self.time_passed))

    def exit_value(self):
        #"""TODO: Generate the exit value for the application."""
        if (self.errors == 0):
            return 0
        else:
            return 42



"""
    YouTube to MPD
    Copyright 2014, Jeroen Doggen, jeroendoggen@gmail.com
"""


import ConfigParser
import argparse
import sys

class Actions:
    create_subfolders = False
    create_playlists = False


class Settings:
    """ Contains all the tools to analyse Blackboard assignments """
    config_file = "settings.conf"
    Config = ConfigParser.ConfigParser()
    playlist = None
    song = None
    music_folder = None
    youtube_foldername = None

    parser = argparse.ArgumentParser(
        prog="youtube_to_mpd",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description="YouTube to MPD commandline arguments:",
        epilog="Report bugs to jeroendoggen@gmail.com.")

    def __init__(self):
        self.read_config_file(self.config_file)
        self.cli_arguments()

    def config_section_map(self, section):
        """ Helper function to read config settings """
        dict1 = {}
        options = self.Config.options(section)
        for option in options:
            try:
                dict1[option] = self.Config.get(section, option)
                if dict1[option] == -1:
                    print("skip: %s" % option)
            except:
                print("exception on %s!" % option)
                dict1[option] = None
                sys.exit(1)
        return dict1

    def read_config_file(self, filename):
        """ Read the config  """
        try:
            self.Config.read(filename)
            self.music_folder = str(self.config_section_map("Config")['music_folder'])
            self.youtube_foldername = str(self.config_section_map("Config")['youtube_foldername'])
            self.create_subfolders = self.config_section_map("Config")['create_subfolders']
            self.create_playlists = self.config_section_map("Config")['create_playlists']

        except AttributeError:
            #TODO: this does not work!! (AttributeError or KeyError needed? both?)
            print("Error while processing config file")
            sys.exit(1)

    def cli_arguments(self):
        """Configure a read all the cli arguments."""
        self.configure_cli_arguments()
        self.get_cli_arguments()

    def configure_cli_arguments(self):
        """Configure all the cli arguments."""
        self.parser.add_argument("-f", metavar="music_folder",
          help="Folder where the music will be saved")
        self.parser.add_argument("-p", metavar="playlist",
          help="Playlist ID (without the http://youtube.com/... part)")
        self.parser.add_argument("-s", metavar="song",
          help="Song ID (without the http://youtube.com/... part)")

        self.parser.add_argument('--create_subfolders', action='store_true', help='Start the virtual machines')
        self.parser.add_argument('--create_playlists', action='store_true', help='Stop the virtual machines')

    def get_cli_arguments(self):
        """Read all the cli arguments."""
        args = self.parser.parse_args()
        if (args.f is not None):
            self.music_folder = args.f
        if (args.p is not None):
            self.playlist = args.p
        if (args.s is not None):
            self.song = args.s

        if (args.create_subfolders is True):
            self.actions.create_subfolders = True
        if (args.create_playlists is True):
            self.actions.create_playlists = True




"""
    YouTube to MPD
    Copyright 2014, Jeroen Doggen, jeroendoggen@gmail.com
"""

import sys
import signal

from youtube_to_mpd import YoutubeToMPD


def run():
    """Run the main program"""
    signal.signal(signal.SIGINT, signal_handler)
    converter = YoutubeToMPD()
    converter.run()
    return(converter.exit_value())

def signal_handler(signal, frame):
        sys.exit(0)

if __name__ == "__main__":
    sys.exit(run())



"""Setup file for YouTube to MPD

Define the options for the "youtube_to_mpd" package
Create source Python packages (python setup.py sdist)
Create binary Python packages (python setup.py bdist)

"""
from distutils.core import setup

from youtube_to_mpd import __version__


with open('README.txt') as readme_file:
    LONG_DESCRIPTION = readme_file.read()

setup(name='youtube_to_mpd',
      version=__version__,
      description='YouTube to MPD',
      long_description=LONG_DESCRIPTION,
      author='Jeroen Doggen',
      author_email='jeroendoggen@gmail.com',
      url='https://github.com/jeroendoggen/youtube_to_mpd',
      packages=['youtube_to_mpd'],
      package_data={'youtube_to_mpd': ['*.py', '*.conf']},
      license='MIT',
      platforms=['Linux'],
      )
